@@ -22,7 +22,6 @@
 
 #include "config.h"
 #include <time.h>
-#include <gconf/gconf-client.h>
 #include <gdk/gdk.h>
 
 #include "gs-prefs.h"        /* for GSSaverMode */
@@ -49,6 +48,7 @@ struct GSManagerPrivate
 
         guint        lock_enabled : 1;
         guint        logout_enabled : 1;
+        guint        throttle_enabled : 1;
         guint        blank_active : 1;
 
         guint        dialog_up : 1;
@@ -60,7 +60,6 @@ struct GSManagerPrivate
 
         GSList      *themes;
         GSSaverMode  saver_mode;
-        GConfClient *gconf_client;
 };
 
 enum {
@@ -77,6 +76,7 @@ enum {
         PROP_CYCLE_TIMEOUT,
         PROP_LOGOUT_TIMEOUT,
         PROP_BLANK_ACTIVE,
+        PROP_THROTTLE_ENABLED,
 };
 
 static GObjectClass *parent_class = NULL;
@@ -112,6 +112,31 @@ gs_manager_set_themes (GSManager *manager,
         }
 }
 
+void
+gs_manager_set_throttle_enabled (GSManager *manager,
+                                 gboolean   throttle_enabled)
+{
+        g_return_if_fail (GS_IS_MANAGER (manager));
+
+        if (manager->priv->throttle_enabled != throttle_enabled) {
+                GSList *l;
+
+                manager->priv->throttle_enabled = throttle_enabled;
+
+                for (l = manager->priv->jobs; l; l = l->next) {
+                        if (throttle_enabled) {
+                                gs_job_stop (GS_JOB (l->data));
+                        } else {
+                                gs_job_start (GS_JOB (l->data));
+                        }
+                }
+
+                for (l = manager->priv->windows; l; l = l->next) {
+                        gs_window_clear (l->data);
+                }
+        }
+}
+
 void
 gs_manager_set_lock_enabled (GSManager *manager,
                              gboolean   lock_enabled)
@@ -212,6 +237,11 @@ select_theme (GSManager *manager)
         if (manager->priv->saver_mode == GS_MODE_BLANK_ONLY)
                 return NULL;
 
+        if (manager->priv->saver_mode == GS_MODE_DONT_BLANK) {
+                g_warning ("Selecting no screensaver theme since screensaver is disabled.");
+                return NULL;
+        }
+
         if (manager->priv->themes) {
                 int number = 0;
 
@@ -241,6 +271,9 @@ gs_manager_cycle (GSManager *manager)
         if (manager->priv->dialog_up)
                 return FALSE;
 
+        if (manager->priv->throttle_enabled)
+                return FALSE;
+
         theme = select_theme (manager);
 
         for (l = manager->priv->jobs; l; l = l->next) {
@@ -309,6 +342,9 @@ gs_manager_set_property (GObject            *object,
         self = GS_MANAGER (object);
 
         switch (prop_id) {
+        case PROP_THROTTLE_ENABLED:
+                gs_manager_set_throttle_enabled (self, g_value_get_boolean (value));
+                break;
         case PROP_LOCK_ENABLED:
                 gs_manager_set_lock_enabled (self, g_value_get_boolean (value));
                 break;
@@ -341,6 +377,9 @@ gs_manager_get_property (GObject            *object,
         self = GS_MANAGER (object);
 
         switch (prop_id) {
+        case PROP_THROTTLE_ENABLED:
+                g_value_set_boolean (value, self->priv->throttle_enabled);
+                break;
         case PROP_LOCK_ENABLED:
                 g_value_set_boolean (value, self->priv->lock_enabled);
                 break;
@@ -444,6 +483,13 @@ gs_manager_class_init (GSManagerClass *klass)
                                                             G_MAXLONG,
                                                             300000,
                                                             G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_THROTTLE_ENABLED,
+                                         g_param_spec_boolean ("throttle-enabled",
+                                                               NULL,
+                                                               NULL,
+                                                               TRUE,
+                                                               G_PARAM_READWRITE));
 
         g_type_class_add_private (klass, sizeof (GSManagerPrivate));
 }
@@ -452,7 +498,6 @@ static void
 gs_manager_init (GSManager *manager)
 {
         manager->priv = GS_MANAGER_GET_PRIVATE (manager);
-        manager->priv->gconf_client = gconf_client_get_default ();
 }
 
 static void
@@ -483,9 +528,6 @@ gs_manager_finalize (GObject *object)
 
         g_return_if_fail (manager->priv != NULL);
 
-        if (manager->priv->gconf_client)
-                g_object_unref (manager->priv->gconf_client);
-
         remove_blank_timers (manager);
 
         gs_grab_release_keyboard_and_mouse ();
@@ -542,10 +584,22 @@ window_dialog_up_cb (GSWindow  *window,
 
         manager->priv->dialog_up = TRUE;
 
-        for (l = manager->priv->jobs; l; l = l->next) {
-                gs_job_suspend (GS_JOB (l->data), TRUE);
+        /* Grab keyboard so dialog can be used */
+        gs_grab_window (gs_window_get_gdk_window (window),
+                        gs_window_get_screen (window),
+                        FALSE);
+
+        /* Make all other windows insensitive so we don't get events */
+        for (l = manager->priv->windows; l; l = l->next) {
+                if (l->data != window)
+                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
         }
 
+        if (! manager->priv->throttle_enabled) {
+                for (l = manager->priv->jobs; l; l = l->next) {
+                        gs_job_suspend (GS_JOB (l->data), TRUE);
+                }
+        }
 }
 
 static void
@@ -557,8 +611,15 @@ window_dialog_down_cb (GSWindow  *window,
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        for (l = manager->priv->jobs; l; l = l->next) {
-                gs_job_suspend (GS_JOB (l->data), FALSE);
+        /* Make all windows sensitive so we get events */
+        for (l = manager->priv->windows; l; l = l->next) {
+                gtk_widget_set_sensitive (GTK_WIDGET (l->data), TRUE);
+        }
+
+        if (! manager->priv->throttle_enabled) {
+                for (l = manager->priv->jobs; l; l = l->next) {
+                        gs_job_suspend (GS_JOB (l->data), FALSE);
+                }
         }
 
         manager->priv->dialog_up = FALSE;
@@ -571,6 +632,8 @@ window_map_event_cb (GSWindow  *window,
 {
         GdkDisplay *display;
         GdkScreen  *screen;
+        int         monitor;
+        int         x, y;
 
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
@@ -578,12 +641,16 @@ window_map_event_cb (GSWindow  *window,
         g_return_val_if_fail (GS_IS_WINDOW (window), FALSE);
 
         display = gdk_display_get_default ();
-        gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
+        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
+        monitor = gdk_screen_get_monitor_at_point (screen, x, y);
 
-        if (gs_window_get_screen (window) == screen)
+        if (gs_window_get_screen (window) == screen
+            && gs_window_get_monitor (window) == monitor) {
                 gs_grab_window (gs_window_get_gdk_window (window),
                                 gs_window_get_screen (window),
                                 FALSE);
+        }
+
         return FALSE;
 }
 
@@ -600,8 +667,7 @@ window_show_cb (GSWindow  *window,
         g_return_if_fail (window != NULL);
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        theme = select_theme (manager);
-        job = gs_job_new_for_widget (GTK_WIDGET (window), theme);
+        job = gs_job_new_for_widget (GTK_WIDGET (window));
 
         /* Add user configuration path */
         /* FIXME: disable this if locked down */
@@ -610,7 +676,13 @@ window_show_cb (GSWindow  *window,
         g_free (path);
 
         g_object_ref (job);
-        gs_job_start (job);
+
+        theme = select_theme (manager);
+        gs_job_set_theme (job, theme, NULL);
+
+        if (! manager->priv->throttle_enabled) {
+                gs_job_start (job);
+        }
 
         manager->priv->jobs = g_slist_append (manager->priv->jobs, job);
 
@@ -634,6 +706,7 @@ window_show_cb (GSWindow  *window,
                                                                  manager);
         }
 
+        /* FIXME: only emit signal once */
         g_signal_emit (manager, signals [BLANKED], 0);
 }
 
@@ -642,6 +715,8 @@ gs_manager_create_window (GSManager *manager,
                           GdkScreen *screen)
 {
         GSWindow *window;
+        int       n_monitors;
+        int       i;
 
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
@@ -649,22 +724,29 @@ gs_manager_create_window (GSManager *manager,
 
         g_object_ref (manager);
         g_object_ref (screen);
-        window = gs_window_new (screen, manager->priv->lock_enabled);
-        gs_window_set_logout_enabled (window, manager->priv->logout_enabled);
-        gs_window_set_logout_timeout (window, manager->priv->logout_timeout);
 
-        g_signal_connect_object (window, "unblanked",
-                                 G_CALLBACK (window_unblanked_cb), manager, 0);
-        g_signal_connect_object (window, "dialog-up",
-                                 G_CALLBACK (window_dialog_up_cb), manager, 0);
-        g_signal_connect_object (window, "dialog-down",
-                                 G_CALLBACK (window_dialog_down_cb), manager, 0);
-        g_signal_connect_object (window, "show",
-                                 G_CALLBACK (window_show_cb), manager, 0);
-        g_signal_connect_object (window, "map_event",
-                                 G_CALLBACK (window_map_event_cb), manager, 0);
+        n_monitors = gdk_screen_get_n_monitors (screen);
+
+        for (i = 0; i < n_monitors; i++) {
+                window = gs_window_new (screen, i, manager->priv->lock_enabled);
+
+                gs_window_set_logout_enabled (window, manager->priv->logout_enabled);
+                gs_window_set_logout_timeout (window, manager->priv->logout_timeout);
+
+                g_signal_connect_object (window, "unblanked",
+                                         G_CALLBACK (window_unblanked_cb), manager, 0);
+                g_signal_connect_object (window, "dialog-up",
+                                         G_CALLBACK (window_dialog_up_cb), manager, 0);
+                g_signal_connect_object (window, "dialog-down",
+                                         G_CALLBACK (window_dialog_down_cb), manager, 0);
+                g_signal_connect_object (window, "show",
+                                         G_CALLBACK (window_show_cb), manager, 0);
+                g_signal_connect_object (window, "map_event",
+                                         G_CALLBACK (window_map_event_cb), manager, 0);
+
+                manager->priv->windows = g_slist_append (manager->priv->windows, window);
+        }
 
-        manager->priv->windows = g_slist_append (manager->priv->windows, window);
         g_object_unref (screen);
         g_object_unref (manager);
 }
@@ -730,7 +812,7 @@ gs_manager_blank (GSManager *manager)
         gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
         root = gdk_screen_get_root_window (screen);
 
-        if (!gs_grab_get_keyboard_and_mouse (root, screen))
+        if (! gs_grab_get_keyboard_and_mouse (root, screen))
                 return FALSE;
 
         if (! manager->priv->windows)
@@ -751,7 +833,7 @@ gs_manager_unblank (GSManager *manager)
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
-        if (!manager->priv->blank_active)
+        if (! manager->priv->blank_active)
                 return FALSE;
 
         remove_blank_timers (manager);
@@ -774,6 +856,7 @@ gs_manager_unblank (GSManager *manager)
         manager->priv->blank_active = FALSE;
         manager->priv->blank_time = 0;
         manager->priv->lock_enabled = FALSE;
+        manager->priv->throttle_enabled = FALSE;
 
         g_signal_emit (manager, signals [UNBLANKED], 0);
 
